
//***********************************************************
// This file has been built using the default PB sample
// included in the standard Orbiter distribution
//***********************************************************

// ==============================================================
//                 ORBITER MODULE: ShuttlePB
//                  Part of the ORBITER SDK
//          Copyright (C) 2002-2003 Martin Schweiger
//                   All rights reserved
//
// ShuttlePB.cpp
// Control module for ShuttlePB vessel class
// ==============================================================

#include "test_ship.h"
#include "include/orbitersdk.h"

// ==============================================================
//
test_ship::test_ship(OBJHANDLE hObj, int fmodel):VESSEL (hObj,fmodel)
{

	Gforce=0.0;
	Separate=-1;Stage=0;

	//initialization of some data
	//1. First we give the SP SDK a pointer to our ship
	
	Panel.RegisterVessel(this);

	//2. Second, we register a custom ship pointer, called  GFORCE
	//   We can then use this name in the CFG file to refer to these specific value

	Panel.RegisterCustomPointer("GFORCE",(void*)&Gforce);Gforce=7.0;

	//a diferent set of rotational values to be used by ADI
	Panel.RegisterCustomPointer("ROLLSET",(void*)&SetRoll);SetRoll=0;
	Panel.RegisterCustomPointer("YAWSET",(void*)&SetYaw);SetYaw=0;
	Panel.RegisterCustomPointer("PITCHSET",(void*)&SetPitch);SetPitch=0;
	Panel.RegisterCustomPointer("HROLL",(void*)&H_roll);H_roll=0;
	Panel.RegisterCustomPointer("HPITCH",(void*)&H_pitch);H_pitch=0;
	Panel.RegisterCustomPointer("HYAW",(void*)&H_yaw);H_yaw=0;
	Panel.RegisterCustomPointer("ADISELECT",(void*)&H_adiselect);H_adiselect=-1;


	//the default roll,pitch,yaw data to be used by the ADI ball
	Panel.RegisterCustomPointer("YAW",(void*)&Yaw);Yaw=0;
	Panel.RegisterCustomPointer("PITCH",(void*)&Pitch);Pitch=0;
	Panel.RegisterCustomPointer("ROLL",(void*)&Roll);Roll=0;

	//3. Lastly, we give the SP SDK the name of the cfg file containing the
	//   configuration script
	Panel.InitFromFile("test_ship");//test_ship.cfg in the config directory. This file containts
									//the name  of the PRD file


	//4. after we have loaded stuff from the config, we can do some extra tweaking
	//such as the re-entry display, that we turn on (it is off by default)
	//note that we "hope" there is a REENTRY instrument defined in panel Main.
	//if not, on will be NULL

	//Eg1
	int *on=(int*)Panel.GetPointerByString("Main:REENTRY:ARM");
	if (on) *on=1;//on

	///similary we could do with some part of electrical or hydraulical systems

	//Eg2
	double *Is_open=(double*)Panel.GetPointerByString("HYDRAULIC:O2TANK_2:OUT:ISOPEN");
	//this will return a pointer to a 0.0 or 1.0 value,
	// signifying wether the OUT valve of the O2TANK_2 is opened or not
	// note that this is a feedback only.It is not a handle. Changing this
	// value will not do anything

	//Eg3
	int *OpenHandle=(int*)Panel.GetPointerByString("HYDRAULIC:O2TANK_2:OUT:OPEN");
	//this on the other hand is the open/close handle, 
	//so 
	if (OpenHandle) *OpenHandle=SP_VALVE_OPEN;	//we can open 
	if (OpenHandle) *OpenHandle=SP_VALVE_CLOSE;	//or close this valve
									

};

// ==============================================================
//
void test_ship::Timestep(double time)
{
	static double last_time;
	double dt=time-last_time;
	last_time=time;

	//update some values, so that things "move" arround the panel 
	Gforce+=0.02;				
	if (Gforce>6.0) Gforce=0;


	//update the ADI data
	oapiGetHeading(this->GetHandle(),&Yaw);
	Pitch=GetPitch();
	Roll=GetBank();

	//update the roll,pitch yaw settings (bottom left of the panel)
	SetRoll+=H_roll*dt*2;
	SetPitch+=H_pitch*dt*2;
	SetYaw+=H_yaw*dt*2;


	//lastly, each timestep is passed to the SP SDK
	// to perform internal computations on the 
	// systems.

	Panel.Timestep(time);
};


// ==============================================================
//
void test_ship::LoadState (FILEHANDLE scn, void *vs)
{
char *line;	
while (oapiReadScenario_nextline (scn, line)) {
	//***************************************************
        if (!strnicmp (line, "<INTERNALS>", 11)) //INTERNALS signals the SPSDK part of the scenario
			Panel.Load(scn);			//send the loading to the Panel
	//***************************************************
		 else if (!strnicmp (line, "STAGE", 5)) //but you are responsible for holding your own data into scn
			sscanf (line+5, "%i", &Stage);		

         else 
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
		}

//***************************************************
Panel.SetStage(Stage,1);// initial stage,1 means we are "loading" into this stage,
						// not separating into it
//***************************************************
};

// ==============================================================
//
void test_ship::SaveState (FILEHANDLE scn)
{
SaveDefaultState (scn);
oapiWriteScenario_int(scn,"STAGE",Stage);

//***************************************************
Panel.Save(scn);	//also save the panel 
//***************************************************
};

// ==============================================================
//
DLLCLBK void ovcLoadStateEx (VESSEL *vessel, FILEHANDLE scn, void *vs)
{
	((test_ship*)vessel)->LoadState (scn, vs);
	
}
// ==============================================================
//
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	((test_ship*)vessel)->SaveState (scn);
}


//**************************************************
//All of the panel events are passed to the PanelSDK

DLLCLBK bool ovcLoadPanel (VESSEL *vessel, int id)
{

	return ((test_ship*)vessel)->Panel.LoadPanel(id);

};

DLLCLBK bool ovcPanelRedrawEvent(VESSEL *vessel,int id, int event, SURFHANDLE surf)
{
 ((test_ship*)(vessel))->Panel.PanelEvent(id,event,surf);

return true;
};

DLLCLBK bool ovcPanelMouseEvent(VESSEL * vessel,int id, int event, int mx, int my)
{ 
 ((test_ship*)(vessel))->Panel.MouseEvent(id,event,mx,my);
	
return true;
}
DLLCLBK void ovcMFDmode(VESSEL *vessel, int mfd, int mode)
{
	((test_ship*)(vessel))->Panel.MFDEvent(mfd);
};

DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	((test_ship*)vessel)->Timestep (simt);
}



